package Übungen.BlackJack;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.jupiter.api.Assertions.*;

import java.util.ArrayList;
import java.util.Collections;

import org.junit.jupiter.api.Test;

class JunitTest {
	//  (expected,actual (welche wert gibt der Methode zurück?),message fehle)

	@Test
	void modellKlasse() {
		// assertEquals(überprüft Parameter miteinander)
		BlackJack t1 = new BlackJack("obai",20);
		BlackJack t2 = new BlackJack("omar",21);
		BlackJackView v1 = new BlackJackView();
		Spiel spie1 = new Spiel(t1,t2,v1);
//		assertEquals("obai", t1.getSpielerName(), "Der Name sollte 'obai' sein.");
//		assertEquals(21,t1.getGuthaben(),"Guthaben habe stimmen nicht");
//		assertNotNull(t1,"es ist nicht null");
		
		// teste arrayList zahlen
//		ArrayList<Integer> kartendeck = new ArrayList<>();
//		for (int i = 1; i <= 11; i++) {
//			kartendeck.add(i);
//		}
//		assertEquals(kartendeck,spie1.getKartendeck());
		
		//aktziptieren nur einen Bedinung die entweder wahr oder true ist
//		assertFalse(spie1.einsatzBestimmen(t1));
//		assertFalse(spie1.einsatzBestimmen(t1));
		
		spie1.zweiKartenZiehen();
	}
	
}
